////////////////////////////////////////////////////////////////////////////////
//
//
//   Author       : Bogdan Popescu
//   Organization : University of Cincinnati
//   Program      : goimf2random
//   Version      : 2.013
//   Date         : Dec 10, 2013
//   Tested       : g++ 4.6.3 x64 (Ubuntu 12.04 LTS x64)   
//                    
//                  
//
//
////////////////////////////////////////////////////////////////////////////////
//
//    MASSCLEAN (MASSive CLuster Evolution and ANalysis Package)
//    Copyright (C) 2007-2013 BOGDAN POPESCU and MARGARET HANSON
//
//    goimf2random.cc is part of MASSCLEAN.
//
//    MASSCLEAN is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation.
//
//    MASSCLEAN is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with MASSCLEAN; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//    or go to : http://www.gnu.org/licenses/gpl.html
//
////////////////////////////////////////////////////////////////////////////////


#include <cstdlib>
#include <ctime>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <iomanip> 



using namespace std;


double N1k(double M, double alpha1, double m1)
{

return (fabs(pow((M/m1),(-alpha1))*M));

}


double N2k(double M, double alpha2, double m1)
{

return (fabs(pow((M/m1),(-alpha2))*M));

}


double N3k(double M, double alpha2, double alpha3, double m1, double m2)
{

return (fabs(pow((m2/m1),(-alpha2))*pow((M/m2),(-alpha3))*M));

}





double N3M(double M, double alpha2, double alpha3, double m1, double m2,double K) 
{
return (fabs(pow((m2/m1),(-alpha2))*pow((M/m2),(-alpha3))*K));

}



double NM1(double M, double alpha1, double m1)
{
return (fabs(pow((M/m1),(-alpha1))));
}

double NM2(double M, double alpha2, double m1)
{
return (fabs(pow((M/m1),(-alpha2))));
}


double NM3(double M, double alpha2, double alpha3, double m1, double m2)
{
return (fabs(pow((m2/m1),(-alpha2))*pow((M/m2),(-alpha3))));


}




int main()
{

  double Mtotal; 
  int VERBOSE;
  
  /// prototypes
  double N1k(double,double,double);
  double N2k(double,double,double);
  double N3k(double,double,double,double,double);
  double N3M(double,double,double,double,double,double);
  
  double NM1(double,double,double);
  double NM2(double,double,double);
  double NM3(double,double,double,double,double);


  cout << "\n";


  cout <<"MASSCLEAN 2.013 (MASSive CLuster Evolution and ANalysis Package)"<<"\n";

  cout <<"Copyright (C) 2007-2013 BOGDAN POPESCU and MARGARET HANSON,"<<"\n";

  cout <<"under GNU General Public License. For details see 'COPYING'."<<"\n";

  cout <<"The program 'goIMF2RANDOM' is part of MASSCLEAN 2.013 BB (Brave Bunny)."<<"\n";

  cout << "\n";
  cout <<"http://www.physics.uc.edu/~popescu/massclean/"<< '\n';

  cout << "\n";


  cout <<"Popescu & Hanson 2009, AJ, 138, 1724"<< '\n';
  cout <<"Popescu & Hanson 2010, ApJL, 713, L21"<< '\n';
  cout <<"Popescu & Hanson 2010, ApJ, 724, 296"<< '\n';
  cout <<"Popescu, Hanson & Elmegreen 2012, ApJ, 751, 122"<< '\n';
  cout <<"Popescu & Hanson 2014, ApJ, 780, 27"<< '\n';


  cout << "\n";
  cout << "\n";

//  sleep(1);
  
  srand((unsigned)time(0));
 
  chdir("ini.files");

  ifstream data0("cluster.ini");

  if (data0.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;
  }

  while (data0.peek()=='#') { data0.ignore(300,'\n');}

  data0 >> VERBOSE; data0.ignore(300,'\n');
  data0 >> Mtotal; data0.ignore(300,'\n');
  data0.ignore(300,'\n');

  data0.close();



  int headonoff;



  //headonoff=0;
  //headonoff=1;


  ifstream data13("header.ini");

  if (data13.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;
  }

  while (data13.peek()=='#') { data13.ignore(300,'\n');}

  data13 >> headonoff; 
  data13.ignore(300,'\n');

  data13.close();


////////////////////////////


cout.setf(ios::fixed,ios::floatfield);

if (VERBOSE==1)
{
  cout <<"Reading file cluster.ini ...";
  cout <<"... done !" <<"\n";
}
  
  cout << "The mass of the cluster is : ";

  cout.precision(2);

  cout <<"M_total =" <<Mtotal<<" M_Sun";

  cout << "\n";

 cout.precision(8);
  
  
   int NOBINS=0;
  
  double m0,m1,m2,m3,mmin,mmax,mtrunc,m4;
  double alpha1,alpha2,alpha3;
  double C1,C2;
  double h1,h2,h3,tol;
  double hn3;
  int noiter;

  int FLUCTUATIONS;

  
  int N1bins,N2bins,N3bins;

  double percentmass;
  
  double MOUT=0.,MOUT2=0.,MOUT3;
  int NOUT;

  
  
  
if (VERBOSE==1)
{
  cout <<"\n";
  cout <<"Reading file imf.ini ...";
}
  
  ifstream data("imf.ini");

  if (data.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;
  }

  while (data.peek()=='#') { data.ignore(300,'\n');}

  data >> m0; data.ignore(300,'\n');
  data >> m1; data.ignore(300,'\n');
  data >> m2; data.ignore(300,'\n');
  data >> m3; data.ignore(300,'\n');
  data >> mmin; data.ignore(300,'\n');
  data >> mmax; data.ignore(300,'\n');
  data >> mtrunc; data.ignore(300,'\n');
  data >> alpha1; data.ignore(300,'\n');
  data >> alpha2; data.ignore(300,'\n');
  data >> alpha3; data.ignore(300,'\n');
/*
  data >> N1bins; data.ignore(300,'\n');
  data >> N2bins; data.ignore(300,'\n');
  data >> N3bins; data.ignore(300,'\n');
  data >> hn3; data.ignore(300,'\n');
  data >> noiter; data.ignore(300,'\n');
  data >> tol; data.ignore(300,'\n');
  
  data >> FLUCTUATIONS; data.ignore(300,'\n');

  data >> percentmass; data.ignore(300,'\n');
*/
  data.ignore(300,'\n');

  data.close();

  percentmass=0.0001;




///////////////////////////////

  double T1,T2,T3,T4,T5,T6;
  double Itotal;

  T1=1./(alpha1-1.)*pow(m0,-alpha1+1)*pow(m1,alpha1);

  T2=-1./(alpha1-1.)*pow(m1,-alpha1+1)*pow(m1,alpha1);

  T3=1./(alpha2-1.)*pow(m1,-alpha2+1)*pow(m1,alpha2);

  T4=-1./(alpha2-1.)*pow(m2,-alpha2+1)*pow(m1,alpha2);

  T5=1./(alpha3-1.)*pow(m2,-alpha3+1)*pow(m1,alpha2)*pow(m2,alpha3-alpha2);

if (mtrunc==-99)
{
  T6=-1./(alpha3-1.)*pow(m3,-alpha3+1)*pow(m1,alpha2)*pow(m2,alpha3-alpha2);
}

if (mtrunc!=-99)
{
T6=0.;
}

double ku;
ku=1.;

Itotal=(T1+T2+T3+T4+T5+T6)*ku;

//cout << Itotal <<"  "<<(T1+T2)*ku<<"  "<<(T3+T4)*ku<<"  "<<(T5+T6)*ku<<'\n';
//cout << Itotal <<"  "<<(T1+T2)*ku<<"  "<<(T1+T2+T3+T4)*ku<<"  "<<(T1+T2+T3+T4+T5+T6)*ku<<'\n';


double u;

double Macumulat,Macumulat2;
double Minclus;

Macumulat=0.;
Minclus=0.;

chdir("..");

 

chdir("data");


ofstream outfile("n_distribution.out");

  if (outfile.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;

  }


  outfile.setf(ios::fixed,ios::floatfield); 

if (headonoff==1)
{
  outfile <<"##  "<< '\n';
  outfile <<"##  MASSCLEAN 2.013 Brave Bunny"<< '\n';
  outfile <<"##  B.Popescu and M.M.Hanson, 2007-2013 "<< '\n';
  outfile <<"##  http://www.physics.uc.edu/~popescu/massclean/"<< '\n';

  outfile <<"##  "<< '\n';

  outfile <<"##  Popescu & Hanson 2009, AJ, 138, 1724"<< '\n';
  outfile <<"##  Popescu & Hanson 2010, ApJL, 713, L21"<< '\n';
  outfile <<"##  Popescu & Hanson 2010, ApJ, 724, 296"<< '\n';
  outfile <<"##  Popescu, Hanson & Elmegreen 2012, ApJ, 751, 122"<< '\n';
  outfile <<"##  Popescu & Hanson 2014, ApJ, 780, 27"<< '\n';

  outfile <<"##  "<< '\n';

  
  outfile <<"# n_distribution.out"<< '\n';
  outfile <<"#"<< '\n';

  outfile.precision(2);
  outfile <<"# The total mass : "<<Mtotal<< '\n';
 

  
  outfile <<"#"<< '\n';
}

int selectie;

selectie=rand()%2;


double M1random,M2random,M3random,Mrandom;

Mrandom=0.;


/////////////////////////////////////////
/// WHILE Macumulat --------------------

//while ((fabs(Macumulat-Mtotal)<=percentmass*Mtotal) || (Macumulat<Mtotal)   )
while (Macumulat<=Mtotal)
{

//////---- generate Mrandom

u=rand()/(RAND_MAX+1.0)*Itotal;
//cout << u <<'\n';


//double M1random,M2random,M3random,Mrandom;

M1random=pow((alpha1-1)*(T1-u)/pow(m1,alpha1),(1./(-alpha1+1)));

M2random=pow((alpha2-1)*(T1+T2+T3-u)/pow(m1,alpha2),(1./(-alpha2+1)));

M3random=pow((alpha3-1)*(T1+T2+T3+T4+T5-u)/(pow(m1,alpha2)*pow(m2,alpha3-alpha2)),(1./(-alpha3+1)));



//cout << M1random<<"  "<< M2random <<"  "<< M3random <<'\n';

if (u<=T1+T2) {Mrandom=M1random;}
if ((u>T1+T2) && (u<=T1+T2+T3+T4)) {Mrandom=M2random;}
if (u>T1+T2+T3+T4) {Mrandom=M3random;}



//////////////Mrandom loop

if ((Mrandom>=m0) && (Mrandom<=mmax))

{



//int selectie;

//selectie=rand()%2;

outfile.precision(8);


//++++++++++++++++++++
if (Macumulat2+Mrandom<=Mtotal*(1+percentmass))
{

if (Mrandom>=0.15)
{
//cout <<"M "<<Mrandom <<'\n';
cout <<setw(12)<< Mrandom <<" M_Sun"<<'\n';

outfile << Mrandom <<"   "<<"1"<<'\n';

Minclus=Minclus+Mrandom;

}

Macumulat2=Macumulat+Mrandom;
Macumulat=Macumulat+Mrandom;
}
//+++++++++++++++++++++++++


//++++++++++++++++++++
if (Macumulat+Mrandom>Mtotal*(1+percentmass))
{
Macumulat2=Macumulat;
Macumulat=Macumulat;
}
//+++++++++++++++++++++++++




}
//////////////Mrandom loop

//if (Macumulat+Mrandom>Mtotal*(1+percentmass)){break;}

////if (Macumulat>Mtotal*(1+percentmass)){break;} ///OK

if ((fabs(Macumulat-Mtotal)<Mtotal*percentmass) &&  (Macumulat>=Mtotal*(1.-percentmass)) ){break;}

}
/// WHILE Macumulat -------------------- END
////////////////////////////////////////////
  
  

outfile.close();


//cout <<rand()%2<<'\n';
  
 
 cout <<"\n";
 cout <<"... done !" <<"\n";
 cout <<"\n";
 
 
 if (VERBOSE==1)
{
  cout <<"Writing file gnuplot_n.gpl ...";
}

  ofstream plotfile("gnuplot_n.gpl");

  if (plotfile.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;

  }

 plotfile <<"set terminal postscript enhanced"<< '\n';
 plotfile <<"set output 'N_"<<Mtotal<<".eps'"<< '\n';

 plotfile <<"#set terminal png"<< '\n';
 plotfile <<"#set output 'N_"<<Mtotal<<".png'"<< '\n';

 plotfile <<"#set size ratio 2"<< '\n';

 plotfile <<"set nokey"<< '\n';


 plotfile <<"set xrange ["<<mmin<<":"<<mmax<<"]"<< '\n';

 plotfile << "set xlabel \" M (solar masses) \""<<'\n';

 plotfile << "set ylabel \" N \""<<'\n';

 plotfile <<"plot 'n_distribution.out' with boxes"<< '\n';

 plotfile.close();

if (VERBOSE==1)
{
 cout <<"... done !" <<"\n";

 cout <<"\n";
 }
 
 chdir("..");
 
if (VERBOSE==1)
{
 cout <<"Writing file plotn ...";
}

  ofstream plotfile2("plotn");

  if (plotfile2.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;

  }
 
 plotfile2 << "cd data"<<'\n';
 plotfile2 << "echo Now plotting..."<<'\n';
 plotfile2 << "gnuplot gnuplot_n.gpl"<<'\n';
 plotfile2 << "echo ...done !"<<'\n';
 
 
 plotfile2.close();

if (VERBOSE==1)
{
 cout <<"... done !" <<"\n";

 cout <<"\n";
 

 cout <<"Writing file plot.all ...";
}

  ofstream plotfile3("plot.all");

  if (plotfile3.bad())
  {
  cerr << "Error ! "<<'\n';
  return 1 ;

  }
 
 plotfile3 << "cd data"<<'\n';
 plotfile3 << "echo       > counterplot"<<'\n';
 plotfile3 << "echo START : >> counterplot"<<'\n';
 plotfile3 << "date >> counterplot"<<'\n';
 plotfile3 << "           >> counterplot"<<'\n';
 plotfile3 << "echo     "<<'\n';
 plotfile3 << "echo Now plotting..."<<'\n';
 plotfile3 << "echo ...please wait..."<<'\n';
 plotfile3 << "echo N Distribution using Kroupa IMF"<<'\n';
 plotfile3 << "gnuplot gnuplot_n.gpl"<<'\n';
 plotfile3 << "echo ...done !"<<'\n';
 plotfile3 << "echo CCM Extinction Model"<<'\n';
 plotfile3 << "gnuplot plotccm.gpl"<<'\n';
 plotfile3 << "echo ...done !"<<'\n';
 plotfile3 << "echo Spatial Distribution using King Model"<<'\n';
 plotfile3 << "gnuplot king.gpl"<<'\n';
 plotfile3 << "echo ...done !"<<'\n';
 plotfile3 << "echo Diagrams : HR and Color-Magnitude Diagrams"<<'\n';
 plotfile3 << "echo ...please wait..."<<'\n';
 plotfile3 << "gnuplot plothr.gpl"<<'\n';
 plotfile3 << "echo ...done !"<<'\n';
 
 
 
 plotfile3 << "echo END : >> counterplot"<<'\n';
 plotfile3 << "date >> counterplot"<<'\n';
 plotfile3 << "echo      >> counterplot"<<'\n';

 plotfile3 << "cat counterplot"<<'\n';
 
 
 
 plotfile3.close();

if (VERBOSE==1)
{
 cout <<"... done !" <<"\n";

 cout <<"\n";
 }
 
 //// ---------------
 
 cout.precision(2);
 
 cout <<"    Mass included in the distribution : "<<Minclus<< " M_Sun"<<"\n";
 
 cout <<"    Percentage of mass included in the distribution : "<<Macumulat2/Mtotal*100.<<"  "<<"%"<<"\n";
 
 cout <<"\n";
 

 
 
  
 






 //chdir("..");
 
 //system("pwd");
 

  
  
   cout <<"... all done !" <<"\n";

 cout <<"\n";



 return 0;
}
